searchNodes=[{"type":"module","title":"cets","doc":"Main CETS module. CETS stores data in-memory. Writes are replicated on all nodes across the cluster. Reads are done locally. This module contains functions to write data. To read data, use an ets module from the Erlang/OTP. The preferred key format is a node specific key (i.e. a key should contain the inserter node name or a pid). This should simplify the cleaning logic. This also avoids the key conflicts during the cluster join. A random key is a good option too. But the cleaning logic in the case of a netsplit could be tricky. Also, if you update a record with a random key, you have to provide a handle_conflict function on startup (because two segments in the cluster could contain a new and an old version of the record, so the records would be overwritten incorrectly). Be careful, CETS does not protect you from records being overwritten. It is a good option to provide handle_conflict function as a start argument, so you could choose, which version of the record to keep, if there are two versions present in the different cluster segments. Often we need to insert some key, if it is not presented yet. Use insert_new for this, it would use a single node to serialize inserts. Check MongooseIM code for examples of usage of this module.","ref":"cets.html"},{"type":"function","title":"cets.delete/2","doc":"Removes an object with the key from all nodes in the cluster. Ideally, nodes should only remove data that they've inserted, not data from another node. See also: delete_many/2 , delete_request/2 .","ref":"cets.html#delete/2"},{"type":"function","title":"cets.delete_many/2","doc":"Removes multiple objects using a list of keys. See also: delete/2 .","ref":"cets.html#delete_many/2"},{"type":"function","title":"cets.delete_many_request/2","doc":"Async delete_many/2 call.","ref":"cets.html#delete_many_request/2"},{"type":"function","title":"cets.delete_object/2","doc":"Removes a specific object. Useful to remove data from ETS tables of type bag .","ref":"cets.html#delete_object/2"},{"type":"function","title":"cets.delete_object_request/2","doc":"Async delete_object/2 call.","ref":"cets.html#delete_object_request/2"},{"type":"function","title":"cets.delete_objects/2","doc":"Removes multiple specific tuples.","ref":"cets.html#delete_objects/2"},{"type":"function","title":"cets.delete_objects_request/2","doc":"Async delete_objects/2 call.","ref":"cets.html#delete_objects_request/2"},{"type":"function","title":"cets.delete_request/2","doc":"Async delete/2 call.","ref":"cets.html#delete_request/2"},{"type":"function","title":"cets.dump/1","doc":"Gets all records from a local ETS table.","ref":"cets.html#dump/1"},{"type":"function","title":"cets.get_leader/1","doc":"Gets the pid of the process, which handles insert_new operations.","ref":"cets.html#get_leader/1"},{"type":"function","title":"cets.get_nodes_request/1","doc":"Async get_nodes/1 call.","ref":"cets.html#get_nodes_request/1"},{"type":"function","title":"cets.info/1","doc":"Returns debug information from the server.","ref":"cets.html#info/1"},{"type":"function","title":"cets.insert/2","doc":"Inserts (or overwrites) a tuple into a table. Only the node that owns the data could update/remove the data. Ideally, Key should contain inserter node info so cleaning and merging is simplified.","ref":"cets.html#insert/2"},{"type":"function","title":"cets.insert_many/2","doc":"Inserts (or overwrites) several tuples into a table.","ref":"cets.html#insert_many/2"},{"type":"function","title":"cets.insert_many_request/2","doc":"Async insert_many/2 call.","ref":"cets.html#insert_many_request/2"},{"type":"function","title":"cets.insert_new/2","doc":"Tries to insert a new record. All inserts are sent to the leader node first. It is a slightly slower comparing to just insert, because extra messaging is required.","ref":"cets.html#insert_new/2"},{"type":"function","title":"cets.insert_new_or_lookup/2","doc":"Inserts a new tuple or returns an existing one.","ref":"cets.html#insert_new_or_lookup/2"},{"type":"function","title":"cets.insert_request/2","doc":"Async insert/2 call.","ref":"cets.html#insert_request/2"},{"type":"function","title":"cets.insert_serial/2","doc":"Serialized version of insert/2 . All insert_serial calls are sent to the leader node first. Similar to insert_new/2 , but overwrites the data silently on conflict. It could be used to update entries, which use not node-specific keys.","ref":"cets.html#insert_serial/2"},{"type":"function","title":"cets.other_nodes/1","doc":"Get a list of other nodes in the cluster that are connected together.","ref":"cets.html#other_nodes/1"},{"type":"function","title":"cets.other_pids/1","doc":"Gets a list of other CETS processes that are handling this table.","ref":"cets.html#other_pids/1"},{"type":"function","title":"cets.pause/1","doc":"Pauses update operations. cets:insert/2 and other functions would block, till the server is unpaused.","ref":"cets.html#pause/1"},{"type":"function","title":"cets.ping/1","doc":"Blocks until all pending Erlang messages are processed by the Server .","ref":"cets.html#ping/1"},{"type":"function","title":"cets.ping_all/1","doc":"Waits till all pending operations are applied.","ref":"cets.html#ping_all/1"},{"type":"function","title":"cets.remote_dump/1","doc":"Gets all records from a remote ETS table.","ref":"cets.html#remote_dump/1"},{"type":"function","title":"cets.set_leader/2","doc":"Sets is_leader field in the state. For debugging only. Setting in in the real life would break leader election logic.","ref":"cets.html#set_leader/2"},{"type":"function","title":"cets.start/2","doc":"Starts a process serving an ETS table. The process would be registered under table_name() name. Options: - handle_down = fun(\#{remote_pid := Pid, table := Tab}) Called when a remote node goes down. This function is called on all nodes in the remaining partition, so you should call the remote nodes from this function. Otherwise a circular locking could happen. i.e. any functions that replicate changes are not allowed (i.e. cets:insert/2 , cets:remove/2 and so on). Use ets module to handle the cleaning (i.e. ets:match_delete/2 ). Use spawn to make a new async process if you need to update the data on the remote nodes, but it could cause an improper cleaning due to the race conditions. - handle_conflict = fun(Record1, Record2) -&gt; NewRecord Called when two records have the same key when clustering. NewRecord would be the record CETS would keep in the table under the key. Does not work for bags. We recommend to define that function if keys could have conflicts. This function would be called once for each conflicting key. We recommend to keep that function pure (or at least no blocking calls from it).","ref":"cets.html#start/2"},{"type":"function","title":"cets.stop/1","doc":"Stops a CETS server.","ref":"cets.html#stop/1"},{"type":"function","title":"cets.table_name/1","doc":"Returns a table name, that the server is serving.","ref":"cets.html#table_name/1"},{"type":"function","title":"cets.unpause/2","doc":"Unpauses update operations. Provide reference, returned from cets:pause/1 as an argument.","ref":"cets.html#unpause/2"},{"type":"function","title":"cets.wait_response/2","doc":"Waits for the result of the async operation.","ref":"cets.html#wait_response/2"},{"type":"function","title":"cets.wait_responses/2","doc":"Waits for multiple responses. Returns results in the same order as ReqIds . Blocks for maximum Timeout milliseconds.","ref":"cets.html#wait_responses/2"},{"type":"type","title":"cets.ack_pid/0","doc":"Pid of the helper process that tracks unacked writes.","ref":"cets.html#t:ack_pid/0"},{"type":"type","title":"cets.backlog_entry/0","doc":"Delayed operation.","ref":"cets.html#t:backlog_entry/0"},{"type":"type","title":"cets.from/0","doc":"gen_server's caller.","ref":"cets.html#t:from/0"},{"type":"type","title":"cets.handle_conflict_fun/0","doc":"Handler function which is called when we need to choose which record to keep during joining.","ref":"cets.html#t:handle_conflict_fun/0"},{"type":"type","title":"cets.handle_down_fun/0","doc":"Handler function which is called when the remote node goes down.","ref":"cets.html#t:handle_down_fun/0"},{"type":"type","title":"cets.handle_wrong_leader/0","doc":"Handler function which is called when a leader operation is received by a non-leader (for debugging).","ref":"cets.html#t:handle_wrong_leader/0"},{"type":"type","title":"cets.info/0","doc":"Status information returned info/1 .","ref":"cets.html#t:info/0"},{"type":"type","title":"cets.join_ref/0","doc":"An unique ID assigned during the table join attempt.","ref":"cets.html#t:join_ref/0"},{"type":"type","title":"cets.long_msg/0","doc":"Types of gen_server calls.","ref":"cets.html#t:long_msg/0"},{"type":"type","title":"cets.node_down_event/0","doc":"","ref":"cets.html#t:node_down_event/0"},{"type":"type","title":"cets.op/0","doc":"Write operation type.","ref":"cets.html#t:op/0"},{"type":"type","title":"cets.pause_monitor/0","doc":"Reference returned from pause/1 .","ref":"cets.html#t:pause_monitor/0"},{"type":"type","title":"cets.remote_op/0","doc":"Message broadcasted to other nodes.","ref":"cets.html#t:remote_op/0"},{"type":"type","title":"cets.request_id/0","doc":"Request Reference.","ref":"cets.html#t:request_id/0"},{"type":"type","title":"cets.response_return/0","doc":"Response return from gen_server 's API. Reply is usually ok.","ref":"cets.html#t:response_return/0"},{"type":"type","title":"cets.response_timeout/0","doc":"Timeout to wait for response.","ref":"cets.html#t:response_timeout/0"},{"type":"type","title":"cets.server_pid/0","doc":"CETS pid.","ref":"cets.html#t:server_pid/0"},{"type":"type","title":"cets.server_ref/0","doc":"CETS Process Reference.","ref":"cets.html#t:server_ref/0"},{"type":"type","title":"cets.servers/0","doc":"Ordered list of server pids.","ref":"cets.html#t:servers/0"},{"type":"type","title":"cets.start_error/0","doc":"","ref":"cets.html#t:start_error/0"},{"type":"type","title":"cets.start_opts/0","doc":"Options for start/2 function.","ref":"cets.html#t:start_opts/0"},{"type":"type","title":"cets.state/0","doc":"","ref":"cets.html#t:state/0"},{"type":"type","title":"cets.table_name/0","doc":"ETS table name (and the process server name).","ref":"cets.html#t:table_name/0"},{"type":"module","title":"cets_ack","doc":"Helper module to keep track of unacked writes. Contains a list of processes, that are waiting for writes to finish. Collects acks from nodes in the cluster. When one of the remote nodes goes down, the server stops waiting for acks from it.","ref":"cets_ack.html"},{"type":"function","title":"cets_ack.ack/3","doc":"Called by a remote server after an operation is applied.","ref":"cets_ack.html#ack/3"},{"type":"function","title":"cets_ack.add/2","doc":"Adds a new task to wait for replies.","ref":"cets_ack.html#add/2"},{"type":"function","title":"cets_ack.send_remote_down/2","doc":"Calls ack(AckPid, From, RemotePid) for all locally tracked From entries.","ref":"cets_ack.html#send_remote_down/2"},{"type":"function","title":"cets_ack.set_servers/2","doc":"Sets a list of servers to be used for newly added tasks.","ref":"cets_ack.html#set_servers/2"},{"type":"function","title":"cets_ack.start_link/1","doc":"Starts a helper process to track unacked writes.","ref":"cets_ack.html#start_link/1"},{"type":"type","title":"cets_ack.ack_pid/0","doc":"Pid of the helper process to track unacked writes.","ref":"cets_ack.html#t:ack_pid/0"},{"type":"type","title":"cets_ack.from/0","doc":"gen_server's caller.","ref":"cets_ack.html#t:from/0"},{"type":"type","title":"cets_ack.server_pid/0","doc":"Pid of CETS gen_server.","ref":"cets_ack.html#t:server_pid/0"},{"type":"type","title":"cets_ack.state/0","doc":"","ref":"cets_ack.html#t:state/0"},{"type":"module","title":"cets_call","doc":"Helper module for extended gen_server API.","ref":"cets_call.html"},{"type":"function","title":"cets_call.async_operation/2","doc":"Contacts the local server to broadcast multinode operation. Returns immediately. You can wait for response from all nodes by calling wait_response/2 .","ref":"cets_call.html#async_operation/2"},{"type":"function","title":"cets_call.long_call/2","doc":"Makes gen_server:call with better error reporting. It would log a warning if the call takes too long.","ref":"cets_call.html#long_call/2"},{"type":"function","title":"cets_call.long_call/3","doc":"Makes gen_server:call with better error reporting.","ref":"cets_call.html#long_call/3"},{"type":"function","title":"cets_call.send_leader_op/2","doc":"Sends all requests to a single node in the cluster.","ref":"cets_call.html#send_leader_op/2"},{"type":"function","title":"cets_call.sync_operation/2","doc":"Contacts the local server to broadcast multinode operation. Blocks until the operation is applied on all nodes.","ref":"cets_call.html#sync_operation/2"},{"type":"function","title":"cets_call.wait_responses/2","doc":"Waits for multiple responses at once.","ref":"cets_call.html#wait_responses/2"},{"type":"type","title":"cets_call.long_msg/0","doc":"Message type.","ref":"cets_call.html#t:long_msg/0"},{"type":"type","title":"cets_call.ok_or_error/0","doc":"Return result.","ref":"cets_call.html#t:ok_or_error/0"},{"type":"type","title":"cets_call.op/0","doc":"Operations are messages which could be buffered when a server is paused. Operations are also broadcasted to the whole cluster.","ref":"cets_call.html#t:op/0"},{"type":"type","title":"cets_call.request_id/0","doc":"Asynchronous request reference.","ref":"cets_call.html#t:request_id/0"},{"type":"type","title":"cets_call.server_ref/0","doc":"Server name or pid.","ref":"cets_call.html#t:server_ref/0"},{"type":"behaviour","title":"cets_discovery","doc":"Node discovery logic. Joins table together when a new node appears. Things that make discovery logic harder: - A table list is dynamic (but eventually we add all the tables into it). - Creating Erlang distribution connection is async, but it net_kernel:ping/1 is blocking. - net_kernel:ping/1 could block for unknown number of seconds (but net_kernel default timeout is 7 seconds). - Resolving nodename could take a lot of time (5 seconds in tests). It is unpredictable blocking. - join tables should be one by one to avoid OOM. - Backend:get_nodes/1 could take a long time. - cets_discovery:get_tables/1 , cets_discovery:add_table/2 should be fast. - The most important net_kernel flags for us to consider are: * dist_auto_connect=never * connect_all * prevent_overlapping_partitions These flags change the way the discovery logic behaves. Also the module would not try to connect to the hidden nodes. Retry logic considerations: - Backend:get_nodes/1 could return an error during startup, so we have to retry fast. - There are two periods of operation for this module: * startup phase, usually first 5 minutes. * regular operation phase, after the startup phase. - We don't need to check for the updated get_nodes too often in the regular operation phase.","ref":"cets_discovery.html"},{"type":"function","title":"cets_discovery.add_table/2","doc":"Adds a table to be tracked and joined.","ref":"cets_discovery.html#add_table/2"},{"type":"function","title":"cets_discovery.delete_table/2","doc":"Deletes a table from being tracked or joined.","ref":"cets_discovery.html#delete_table/2"},{"type":"callback","title":"cets_discovery.get_nodes/1","doc":"","ref":"cets_discovery.html#c:get_nodes/1"},{"type":"function","title":"cets_discovery.get_tables/1","doc":"Gets a list of the tracked tables.","ref":"cets_discovery.html#get_tables/1"},{"type":"function","title":"cets_discovery.info/1","doc":"Gets information for each tracked table.","ref":"cets_discovery.html#info/1"},{"type":"callback","title":"cets_discovery.init/1","doc":"","ref":"cets_discovery.html#c:init/1"},{"type":"function","title":"cets_discovery.start/1","doc":"Starts a discovery process.","ref":"cets_discovery.html#start/1"},{"type":"function","title":"cets_discovery.start_link/1","doc":"Starts a discovery process with a link.","ref":"cets_discovery.html#start_link/1"},{"type":"function","title":"cets_discovery.system_info/1","doc":"Gets discovery process status.","ref":"cets_discovery.html#system_info/1"},{"type":"function","title":"cets_discovery.wait_for_get_nodes/2","doc":"Waits for the current get_nodes call to return. Just returns if there is no gen_nodes call running. Waits for another get_nodes, if should_retry_get_nodes flag is set. It is different from wait_for_ready, because it does not wait for unavailable nodes to return pang.","ref":"cets_discovery.html#wait_for_get_nodes/2"},{"type":"function","title":"cets_discovery.wait_for_ready/2","doc":"Blocks until the initial discovery is done. This call would also wait till the data is loaded from the remote nodes.","ref":"cets_discovery.html#wait_for_ready/2"},{"type":"type","title":"cets_discovery.backend_state/0","doc":"Backend state.","ref":"cets_discovery.html#t:backend_state/0"},{"type":"type","title":"cets_discovery.from/0","doc":"gen_server's caller.","ref":"cets_discovery.html#t:from/0"},{"type":"type","title":"cets_discovery.get_nodes_result/0","doc":"Result of get_nodes/2 call.","ref":"cets_discovery.html#t:get_nodes_result/0"},{"type":"type","title":"cets_discovery.join_result/0","doc":"Join result information.","ref":"cets_discovery.html#t:join_result/0"},{"type":"type","title":"cets_discovery.milliseconds/0","doc":"Number of milliseconds.","ref":"cets_discovery.html#t:milliseconds/0"},{"type":"type","title":"cets_discovery.opts/0","doc":"Backend could define its own options.","ref":"cets_discovery.html#t:opts/0"},{"type":"type","title":"cets_discovery.retry_type/0","doc":"Retry logic type.","ref":"cets_discovery.html#t:retry_type/0"},{"type":"type","title":"cets_discovery.server/0","doc":"Discovery server process.","ref":"cets_discovery.html#t:server/0"},{"type":"type","title":"cets_discovery.start_result/0","doc":"Result of start_link/1 .","ref":"cets_discovery.html#t:start_result/0"},{"type":"type","title":"cets_discovery.state/0","doc":"","ref":"cets_discovery.html#t:state/0"},{"type":"type","title":"cets_discovery.system_info/0","doc":"Discovery status.","ref":"cets_discovery.html#t:system_info/0"},{"type":"module","title":"cets_discovery_file","doc":"File backend for cets_discovery. Barebone AWS EC2 auto-discovery is limited: - UDP broadcasts do not work. - AWS CLI needs access. - DNS does not allow to list subdomains. So, we use a file with nodes to connect as a discovery mechanism (so, you can hardcode nodes or use your method of filling it).","ref":"cets_discovery_file.html"},{"type":"function","title":"cets_discovery_file.get_nodes/1","doc":"Get a list of nodes.","ref":"cets_discovery_file.html#get_nodes/1"},{"type":"function","title":"cets_discovery_file.init/1","doc":"Init backend.","ref":"cets_discovery_file.html#init/1"},{"type":"type","title":"cets_discovery_file.opts/0","doc":"Start options.","ref":"cets_discovery_file.html#t:opts/0"},{"type":"type","title":"cets_discovery_file.state/0","doc":"The backend state.","ref":"cets_discovery_file.html#t:state/0"},{"type":"module","title":"cets_dist_blocker","doc":"Disallow distributed erlang connections until cleaning is done. This module prevents a node from reconnecting, until cleaning activity is finished. It prevents race conditions. This module assumes all nodes share the same cookie.","ref":"cets_dist_blocker.html"},{"type":"function","title":"cets_dist_blocker.add_cleaner/1","doc":"Register CleanerPid as a cleaner.","ref":"cets_dist_blocker.html#add_cleaner/1"},{"type":"function","title":"cets_dist_blocker.cleaning_done/2","doc":"Confirm that cleaning is done. This function should be called by a cleaner when it receives nodedown and finishes cleaning.","ref":"cets_dist_blocker.html#cleaning_done/2"},{"type":"function","title":"cets_dist_blocker.code_change/3","doc":"","ref":"cets_dist_blocker.html#code_change/3"},{"type":"function","title":"cets_dist_blocker.handle_call/3","doc":"","ref":"cets_dist_blocker.html#handle_call/3"},{"type":"function","title":"cets_dist_blocker.handle_cast/2","doc":"","ref":"cets_dist_blocker.html#handle_cast/2"},{"type":"function","title":"cets_dist_blocker.handle_info/2","doc":"","ref":"cets_dist_blocker.html#handle_info/2"},{"type":"function","title":"cets_dist_blocker.init/1","doc":"","ref":"cets_dist_blocker.html#init/1"},{"type":"function","title":"cets_dist_blocker.start_link/0","doc":"Spawn dist_blocker","ref":"cets_dist_blocker.html#start_link/0"},{"type":"function","title":"cets_dist_blocker.terminate/2","doc":"","ref":"cets_dist_blocker.html#terminate/2"},{"type":"type","title":"cets_dist_blocker.cleaner_pid/0","doc":"","ref":"cets_dist_blocker.html#t:cleaner_pid/0"},{"type":"type","title":"cets_dist_blocker.state/0","doc":"","ref":"cets_dist_blocker.html#t:state/0"},{"type":"type","title":"cets_dist_blocker.waiting/0","doc":"","ref":"cets_dist_blocker.html#t:waiting/0"},{"type":"module","title":"cets_join","doc":"Cluster join logic. Join is called for each table by the discovery process. Checkpoints are used for testing and do not affect the joining logic.","ref":"cets_join.html"},{"type":"function","title":"cets_join.join/4","doc":"Join two clusters with default options.","ref":"cets_join.html#join/4"},{"type":"function","title":"cets_join.join/5","doc":"Join two clusters. Writes would wait for join completion.","ref":"cets_join.html#join/5"},{"type":"function","title":"cets_join.pause_on_remote_node/2","doc":"","ref":"cets_join.html#pause_on_remote_node/2"},{"type":"type","title":"cets_join.checkpoint/0","doc":"Critical events during the joining procedure.","ref":"cets_join.html#t:checkpoint/0"},{"type":"type","title":"cets_join.checkpoint_handler/0","doc":"Checkpoint function for debugging.","ref":"cets_join.html#t:checkpoint_handler/0"},{"type":"type","title":"cets_join.join_opts/0","doc":"Joining options.","ref":"cets_join.html#t:join_opts/0"},{"type":"type","title":"cets_join.join_ref/0","doc":"An unique ID assigned during the table join attempt.","ref":"cets_join.html#t:join_ref/0"},{"type":"type","title":"cets_join.lock_key/0","doc":"Key for global:trans/4 .","ref":"cets_join.html#t:lock_key/0"},{"type":"type","title":"cets_join.rpc_result/0","doc":"Pid of CETS gen_server.","ref":"cets_join.html#t:rpc_result/0"},{"type":"type","title":"cets_join.server_pid/0","doc":"","ref":"cets_join.html#t:server_pid/0"},{"type":"module","title":"cets_long","doc":"Helper to log long running operations.","ref":"cets_long.html"},{"type":"function","title":"cets_long.run_spawn/2","doc":"Spawns a new process to do some memory-intensive task. This allows to reduce GC on the parent process. Waits for function to finish. Handles errors. Returns result from the function or crashes (i.e. forwards an error).","ref":"cets_long.html#run_spawn/2"},{"type":"function","title":"cets_long.run_tracked/2","doc":"Runs function Fun . Logs errors. Logs if function execution takes too long. Does not catches the errors - the caller would have to catch if they want to prevent an error.","ref":"cets_long.html#run_tracked/2"},{"type":"type","title":"cets_long.log_info/0","doc":"Extra logging information.","ref":"cets_long.html#t:log_info/0"},{"type":"type","title":"cets_long.task_fun/0","doc":"User provided function to execute.","ref":"cets_long.html#t:task_fun/0"},{"type":"type","title":"cets_long.task_result/0","doc":"The generic result of execution.","ref":"cets_long.html#t:task_result/0"},{"type":"module","title":"cets_metadata","doc":"A helper module for fast access for some server metadata.","ref":"cets_metadata.html"},{"type":"function","title":"cets_metadata.get/2","doc":"Reads metadata.","ref":"cets_metadata.html#get/2"},{"type":"function","title":"cets_metadata.init/1","doc":"Create a table for fast access.","ref":"cets_metadata.html#init/1"},{"type":"function","title":"cets_metadata.set/3","doc":"Sets metadata.","ref":"cets_metadata.html#set/3"},{"type":"module","title":"cets_ping","doc":"Helper module for pinging remote nodes.","ref":"cets_ping.html"},{"type":"function","title":"cets_ping.ping/1","doc":"Pings a remote node.","ref":"cets_ping.html#ping/1"},{"type":"function","title":"cets_ping.ping_pairs/1","doc":"Run multiple ping requests from many nodes.","ref":"cets_ping.html#ping_pairs/1"},{"type":"function","title":"cets_ping.send_ping_result/3","doc":"Sends ping result back to cets_discovery.","ref":"cets_ping.html#send_ping_result/3"},{"type":"module","title":"cets_status","doc":"Status of all tables.","ref":"cets_status.html"},{"type":"function","title":"cets_status.format_data/1","doc":"Formats gathered data. Contains pure functions logic.","ref":"cets_status.html#format_data/1"},{"type":"function","title":"cets_status.gather_data/1","doc":"Collects status information.","ref":"cets_status.html#gather_data/1"},{"type":"function","title":"cets_status.get_local_table_to_other_nodes_map_from_disco/1","doc":"","ref":"cets_status.html#get_local_table_to_other_nodes_map_from_disco/1"},{"type":"function","title":"cets_status.status/1","doc":"Collects and analyses status information.","ref":"cets_status.html#status/1"},{"type":"type","title":"cets_status.disco_name/0","doc":"Discovery process name (atom).","ref":"cets_status.html#t:disco_name/0"},{"type":"type","title":"cets_status.info/0","doc":"","ref":"cets_status.html#t:info/0"},{"type":"type","title":"cets_status.node_to_tab_nodes_map/0","doc":"Mapping from a node name to tab_nodes_map .","ref":"cets_status.html#t:node_to_tab_nodes_map/0"},{"type":"type","title":"cets_status.status_data/0","doc":"Intermediate status information.","ref":"cets_status.html#t:status_data/0"},{"type":"type","title":"cets_status.tab_nodes_map/0","doc":"Mapping from a table name to a node list.","ref":"cets_status.html#t:tab_nodes_map/0"},{"type":"type","title":"cets_status.table_name/0","doc":"Table name (atom).","ref":"cets_status.html#t:table_name/0"},{"type":"extras","title":"Cluster ETS","doc":"The project adds replication support for Erlang Term Storage (ETS). It allows to insert or delete objects into ETS tables across several Erlang nodes. The closest comparison is Mnesia with dirty operations. Some features are not supported: there is no schema there is no persistency there are no indexes there are no transactions (there are bulk inserts/deletes instead). Developed by Erlang Solutions to be used in MongooseIM . It could be used in other Erlang or Elixir projects. Merging logic When two database partitions are joined together, records from both partitions are put together. So, each record would be present on each node. The idea that each node updates only records that it owns. The node name should be inside an ETS key for this to work (or a pid). When some node is down, we remove all records that are owned by this node. When a node reappears, the records are added back. You can also use totally random keys. In this case all keys should be unique. We also support tables with type=bag. In such case all records would remain in the table. You can provide the handle_conflict function, which would be called if two records have the same key when clustering (just be aware, this function would not protect you against overwrites by the same node). Without handle_conflict , conflicting records would overwrite each other when joining. When joining nodes we insert all data from one node to another and vice versa without any extra check. Check join_works_with_existing_data_with_conflicts test case as an example. API The main module is cets. It exports functions: start(Tab, Opts) - starts a new table manager. There is one gen_server for each node for each table. insert(Server, Rec) - inserts a new object into a table. insert_many(Server, Records) - inserts several objects into a table. delete(Server, Key) - deletes an object from the table. delete_many(Server, Keys) - deletes several objects from the table. cets_join module contains the merging logic. cets_discovery module handles search of new nodes. It supports behaviours for different backends. It defines two callbacks: init/1 - inits the backend. get_nodes/1 - gets a list of alive erlang nodes. Once new nodes are found, cets_discovery calls cets_join module to merge two cluster partitions. The simplest cets_discovery backend is cets_discovery_file , which just reads a file with a list of nodes on each line. This file could be populated by an external program or by an admin. Commands Before making a new pull request run tests: rebar3 all","ref":"readme.html"},{"type":"extras","title":"LICENSE","doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html"}]